//---------Copyright (C) 1997 Dallas Semiconductor Corporation--------------
//
//  IBTYPES.H   - include file containing the standard type for iButton 
//                TMEX.
//
//  Compiler: VC
//  Version: 3.10
//  Revisions:
//

// ------------------- Type Defs ---------------------------------------------
typedef unsigned char  uchar;
typedef unsigned short ushort;
typedef unsigned long  ulong;

// internal file information structure
typedef struct 
{
   uchar Name[4];          // 4  File/Directory name
   uchar Ext;              // 1  File/Directory extension
   uchar Spage;            // 1  Start page
   uchar NumPgs;           // 1  Number of pages
   uchar Attrib;           // 1  Attribute 0x01 for readonly 0x02 for hidden
   uchar Dpage;            // 1  Directory page this entry is in
   uchar Offset;           // 1  Byte offset shorto directory page of entry
   uchar Read;             // 1  Read = 1 if opened and 0 if created 
   uchar PDpage;           // 1  Previous directory page number
} FileInfo;

// external file information structure
typedef struct 
{
   uchar Name[4];          // 4
   uchar Ext;              // 1
   uchar Spage;            // 1
   uchar NumPgs;           // 1
   uchar Attrib;           // 1
   uchar BM[32];           // 32
} FileEntry;
               
// structure use when reading or changing the current directory               
typedef struct
{
   uchar NumEntries;      // number of entries in path 0-10 
   char  Ref;             // reference character '\' or '.' 
   char  Entries[10][4];  // sub-directory entry names                                           
} DirectoryPath;

// current directory structure
typedef struct 
{
   uchar ne;          // number of entries
   struct {
      uchar name[4];  // name and extention of the sub-directory
      uchar page;     // page the subdirectory starts on  
      uchar attrib;   // attribute of the directory
   } Entry [10];
   uchar DRom[8];     // rom that this directory is valid for
} CurrentDirectory;    

// structure to hold the program job list ~ 7977 bytes                            
typedef struct
{                    
   uchar ERom[8];     // rom of the program job target
   uchar EBitmap[32]; // bit map kept for 
   uchar OrgBitmap[32]; // original bitmap
   struct {
      uchar wr;       // flag to indicate this page needs to be written
      uchar len;      // length of data
      uchar data[29]; // data to write 
      uchar bm[4];    // bitmap for bytes to write 
   } Page[256];       // 256 possible jobs
} ProgramJob;                             

// structure of garbage bag    
typedef struct 
{
   // used by network layer
   uchar LD;              // 1
   uchar DOWCRC;          // 1
   ushort CRC16;          // 2
   uchar ROM[8];          // 8    
   uchar LF;              // 1
   uchar AutoOverDrive;   // 1  (3.00)   
   uchar LastOne;         // 1         

   // used by file layer
   uchar LastFile;        // 1  (2.00)
   FileInfo Han[5];       // 60 (2.00)
   CurrentDirectory CD;   // 69 (2.00)               
   uchar Internal;        // 1  (2.00)                             
   // Page_WR in file layer
   uchar LastPage;        // 1
   uchar FPage;           // 1  
   ushort  buflen;        // 2
   uchar pgcnt;           // 1
   char  mxpgs;           // 1  
   char  BadRec;          // 1

   // used by transport layer
   uchar FastMode;        // 2 

} Garbage_Bag;
         
// structure of global variables at the head buffer before the
// garbage bag  155 bytes so total = 35 + 155 * 16 = 2515  
// regular or 2515+7977 = 10492 for Program                     
typedef struct
{
   uchar  JFlag;           // 1  Program job flag      
   ushort INIT_FLAG;       // flag for buffer initialization
   struct 
   {
      char Type;          // structure to hold Garbage Bag Map
      char Port;
   } GBMap[16];                
   
} Global_Bag;                      

// size of Garbage Bag buffer
#define SIZEGB     sizeof(Garbage_Bag)
#define SIZEGLOBAL sizeof(Global_Bag)

// structure of the search state    
typedef struct 
{
   // used by network layer
   uchar ROM[8];          // 8    
   uchar LD;              // 1
   uchar LF;              // 1
   uchar LastOne;         // 1         

} SearchState;
